@@ -18,6 +18,11 @@ struct BUSHeader {
   std::string text;
   std::vector<BUSTranscript> transcripts;
   std::vector<std::vector<int32_t>> ecs;
+  uint32_t version;
+  uint32_t bclen;
+  uint32_t umilen;
+  BUSHeader() : version(0), bclen(0), umilen(0) {}
+
 };
 
 struct BUSData {
@@ -26,8 +31,8 @@ struct BUSData {
   int32_t ec;
   uint32_t count;
   uint32_t flags;  
-
-  BUSData() : barcode(0), UMI(0), ec(-1), count(0), flags(0) {}
+  uint32_t pad;
+  BUSData() : barcode(0), UMI(0), ec(-1), count(0), flags(0), pad(0) {}
 };
 
 uint64_t stringToBinary(const std::string &s, uint32_t &flag);
